==> jaxx-widgets-gis-2.16.1-sources.jar: org/nuiton/jaxx/widgets/gis/absolute/AbsoluteDmdCoordinateEditorHandler.java <==
package org.nuiton.jaxx.widgets.gis.absolute;

/*
 * #%L
 * JAXX :: Widgets Gis
 * %%
 * Copyright (C) 2008 - 2014 CodeLutin
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Lesser Public License for more details.
 * 
 * You should have received a copy of the GNU General Lesser Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/lgpl-3.0.html>.
 * #L%
 */

import com.google.common.base.Preconditions;
import jaxx.runtime.spi.UIHandler;
import org.nuiton.jaxx.widgets.gis.DmdCoordinate;
import org.nuiton.jaxx.widgets.gis.DmdCoordinateConverter;
import org.nuiton.jaxx.widgets.gis.MaskFormatterFromConverter;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.nuiton.util.beans.BeanUtil;

import javax.swing.JFormattedTextField;
import javax.swing.text.DefaultFormatterFactory;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.lang.reflect.Method;
import java.text.ParseException;

/**
 * Created on 9/2/14.
 *
 * @author Tony Chemit - chemit@codelutin.com
 * @since 2.12
 */
public class AbsoluteDmdCoordinateEditorHandler implements UIHandler<AbsoluteDmdCoordinateEditor> {

    /** Logger. */
    private static final Log log =
            LogFactory.getLog(AbsoluteDmdCoordinateEditorHandler.class);

    private AbsoluteDmdCoordinateEditor ui;

    protected Method signMutator;

    protected Method degreeMutator;

    protected Method minuteMutator;

    protected Method decimalMutator;

    protected boolean valueIsAdjusting;

    protected boolean valueModelIsAdjusting;

    protected DmdCoordinateConverter unsignedConverter;

    protected DefaultFormatterFactory formatterFactory;

    @Override
    public void beforeInit(AbsoluteDmdCoordinateEditor ui) {
        this.ui = ui;
        // can't use the one from ConverterUtil since we deal with some internal states
        this.unsignedConverter = new DmdCoordinateConverter();
    }

    @Override
    public void afterInit(AbsoluteDmdCoordinateEditor ui) {
        // nothing special to do here
    }

    public void init(boolean longitudeEditor) {

        final AbsoluteDmdCoordinateEditorModel model = ui.getModel();

        Preconditions.checkNotNull(model.getBean(), "could not find bean in " + ui);
        Preconditions.checkNotNull(model.getPropertyDegree(), "could not find propertyDegree in " + ui);
        Preconditions.checkNotNull(model.getPropertyMinute(), "could not find propertyMinute in " + ui);
        Preconditions.checkNotNull(model.getPropertyDecimal(), "could not find propertyDecimal in " + ui);

        Object bean = model.getBean();

        degreeMutator = BeanUtil.getMutator(bean, model.getPropertyDegree());
        Preconditions.checkNotNull(degreeMutator, "could not find mutator for " + model.getPropertyDegree());

        minuteMutator = BeanUtil.getMutator(bean, model.getPropertyMinute());
        Preconditions.checkNotNull(minuteMutator, "could not find mutator for " + model.getPropertyMinute());

        decimalMutator = BeanUtil.getMutator(bean, model.getPropertyDecimal());
        Preconditions.checkNotNull(decimalMutator, "could not find mutator for " + model.getPropertyDecimal());

        unsignedConverter.setForLongitude(longitudeEditor);

        {
            // prepare unsigned formatter factory
            String pattern = getMaskFormatterPattern(longitudeEditor);
            MaskFormatterFromConverter<DmdCoordinate> maskFormatter;
            try {
                maskFormatter = MaskFormatterFromConverter.newFormatter(
                        DmdCoordinate.class,
                        pattern, unsignedConverter);
                maskFormatter.setValidCharacters(" 01234567890");
                maskFormatter.setCommitsOnValidEdit(true);
                formatterFactory = new DefaultFormatterFactory(maskFormatter);
            } catch (ParseException e) {
                // can't happen here
                throw new RuntimeException(e);
            }
        }

        JFormattedTextField editor = ui.getEditor();
        editor.setFormatterFactory(formatterFactory);
        editor.setFocusLostBehavior(JFormattedTextField.COMMIT);

        // When editor changes his value, propagate it to model
        editor.addPropertyChangeListener("value", new PropertyChangeListener() {
            @Override
            public void propertyChange(PropertyChangeEvent evt) {
                DmdCoordinate newValue = (DmdCoordinate) evt.getNewValue();
                if (log.isDebugEnabled()) {
                    log.debug("Value has changed: " + newValue);
                }
                model.setValue(newValue);
            }
        });

        // When model degre changed, let's push it back in bean
        model.addPropertyChangeListener(
                AbsoluteDmdCoordinateEditorModel.PROPERTY_DEGREE,
                new ModelPropertyChangeListener(model, degreeMutator));

        // When model minute changed, let's push it back in bean
        model.addPropertyChangeListener(
                AbsoluteDmdCoordinateEditorModel.PROPERTY_MINUTE,
                new ModelPropertyChangeListener(model, minuteMutator));

        // When model decimal changed, let's push it back in bean
        model.addPropertyChangeListener(
                AbsoluteDmdCoordinateEditorModel.PROPERTY_DECIMAL,
                new ModelPropertyChangeListener(model, decimalMutator));
    }

    public void setValue(DmdCoordinate value, boolean pushToModel) {

        if (valueModelIsAdjusting) {
            // avoid re-entrant code
            return;
        }

        valueIsAdjusting = !pushToModel;

        try {
            ui.getEditor().setValue(value);
        } finally {
            valueIsAdjusting = false;
        }
    }

    public void resetEditor() {
        // set null value to model
        setValue(null, true);

        // use back unsigned format
        ui.getEditor().setFormatterFactory(formatterFactory);
    }

    protected String getMaskFormatterPattern(boolean longitudeEditor) {
        String pattern = "**°**''**";
        if (longitudeEditor) {
            // add one more degre
            pattern = "*" + pattern;
        }
        return pattern;
    }

    private class ModelPropertyChangeListener implements PropertyChangeListener {

        private final AbsoluteDmdCoordinateEditorModel model;

        private final Method mutator;

        private ModelPropertyChangeListener(AbsoluteDmdCoordinateEditorModel model, Method mutator) {
            this.model = model;
            this.mutator = mutator;
        }

        @Override
        public void propertyChange(PropertyChangeEvent evt) {
            if (!valueIsAdjusting) {
                Object newValue = evt.getNewValue();

                try {

                    valueModelIsAdjusting = true;
                    try {
                        mutator.invoke(model.getBean(), newValue);
                    } finally {
                        valueModelIsAdjusting = false;
                    }

                } catch (Exception e) {
                    throw new RuntimeException(e);
                }
            }
        }
    }
}

==> jaxx-widgets-gis-2.20.1-sources.jar: org/nuiton/jaxx/widgets/gis/absolute/AbsoluteDmdCoordinateEditorHandler.java <==
package org.nuiton.jaxx.widgets.gis.absolute;

/*
 * #%L
 * JAXX :: Widgets Gis
 * %%
 * Copyright (C) 2008 - 2014 CodeLutin
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Lesser Public License for more details.
 * 
 * You should have received a copy of the GNU General Lesser Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/lgpl-3.0.html>.
 * #L%
 */

import com.google.common.base.Preconditions;
import jaxx.runtime.spi.UIHandler;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.nuiton.jaxx.widgets.gis.DmdCoordinate;
import org.nuiton.jaxx.widgets.gis.DmdCoordinateConverter;
import org.nuiton.jaxx.widgets.gis.MaskFormatterFromConverter;
import org.nuiton.util.beans.BeanUtil;

import javax.swing.JFormattedTextField;
import javax.swing.text.DefaultFormatterFactory;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.lang.reflect.Method;
import java.text.ParseException;

/**
 * Created on 9/2/14.
 *
 * @author Tony Chemit - chemit@codelutin.com
 * @since 2.12
 */
public class AbsoluteDmdCoordinateEditorHandler implements UIHandler<AbsoluteDmdCoordinateEditor> {

    /** Logger. */
    private static final Log log =
            LogFactory.getLog(AbsoluteDmdCoordinateEditorHandler.class);

    private AbsoluteDmdCoordinateEditor ui;

    protected Method signMutator;

    protected Method degreeMutator;

    protected Method minuteMutator;

    protected Method decimalMutator;

    protected boolean valueIsAdjusting;

    protected boolean valueModelIsAdjusting;

    protected DmdCoordinateConverter coordinateConverter;

    @Override
    public void beforeInit(AbsoluteDmdCoordinateEditor ui) {
        this.ui = ui;
        // can't use the one from ConverterUtil since we deal with some internal states
        this.coordinateConverter = new DmdCoordinateConverter();
    }

    @Override
    public void afterInit(AbsoluteDmdCoordinateEditor ui) {
        // nothing special to do here
    }

    public void init(boolean longitudeEditor) {

        final AbsoluteDmdCoordinateEditorModel model = ui.getModel();

        Preconditions.checkNotNull(model.getBean(), "could not find bean in " + ui);
        Preconditions.checkNotNull(model.getPropertyDegree(), "could not find propertyDegree in " + ui);
        Preconditions.checkNotNull(model.getPropertyMinute(), "could not find propertyMinute in " + ui);
        Preconditions.checkNotNull(model.getPropertyDecimal(), "could not find propertyDecimal in " + ui);

        Object bean = model.getBean();

        degreeMutator = BeanUtil.getMutator(bean, model.getPropertyDegree());
        Preconditions.checkNotNull(degreeMutator, "could not find mutator for " + model.getPropertyDegree());

        minuteMutator = BeanUtil.getMutator(bean, model.getPropertyMinute());
        Preconditions.checkNotNull(minuteMutator, "could not find mutator for " + model.getPropertyMinute());

        decimalMutator = BeanUtil.getMutator(bean, model.getPropertyDecimal());
        Preconditions.checkNotNull(decimalMutator, "could not find mutator for " + model.getPropertyDecimal());

        coordinateConverter.setForLongitude(longitudeEditor);

        // prepare unsigned formatter factory
        String pattern = getMaskFormatterPattern(longitudeEditor);
        MaskFormatterFromConverter<DmdCoordinate> maskFormatter;
        try {
            maskFormatter = MaskFormatterFromConverter.newFormatter(
                    DmdCoordinate.class,
                    pattern, coordinateConverter);
            maskFormatter.setValidCharacters(" 01234567890");
            maskFormatter.setCommitsOnValidEdit(true);

        } catch (ParseException e) {
            // can't happen here
            throw new RuntimeException(e);
        }

        JFormattedTextField editor = ui.getEditor();

        DefaultFormatterFactory formatterFactory = new DefaultFormatterFactory(maskFormatter);
        editor.setFormatterFactory(formatterFactory);
        editor.setFocusLostBehavior(JFormattedTextField.COMMIT);

        // When editor changes his value, propagate it to model
        editor.addPropertyChangeListener("value", new PropertyChangeListener() {
            @Override
            public void propertyChange(PropertyChangeEvent evt) {
                DmdCoordinate newValue = (DmdCoordinate) evt.getNewValue();
                if (log.isDebugEnabled()) {
                    log.debug("Value has changed: " + newValue);
                }
                model.setValue(newValue);
            }
        });

        // When model degre changed, let's push it back in bean
        model.addPropertyChangeListener(
                AbsoluteDmdCoordinateEditorModel.PROPERTY_DEGREE,
                new ModelPropertyChangeListener(model, degreeMutator));

        // When model minute changed, let's push it back in bean
        model.addPropertyChangeListener(
                AbsoluteDmdCoordinateEditorModel.PROPERTY_MINUTE,
                new ModelPropertyChangeListener(model, minuteMutator));

        // When model decimal changed, let's push it back in bean
        model.addPropertyChangeListener(
                AbsoluteDmdCoordinateEditorModel.PROPERTY_DECIMAL,
                new ModelPropertyChangeListener(model, decimalMutator));
    }

    public void setValue(DmdCoordinate value, boolean pushToModel) {

        if (valueModelIsAdjusting) {
            // avoid re-entrant code
            return;
        }

        valueIsAdjusting = !pushToModel;

        try {
            ui.getEditor().setValue(value);
        } finally {
            valueIsAdjusting = false;
        }
    }

    public void resetEditor() {

        // set null value to model
        setValue(null, true);

    }

    protected String getMaskFormatterPattern(boolean longitudeEditor) {
        String pattern = "**°**.**''";
        if (longitudeEditor) {
            // add one more degre
            pattern = "*" + pattern;
        }
        return pattern;
    }

    private class ModelPropertyChangeListener implements PropertyChangeListener {

        private final AbsoluteDmdCoordinateEditorModel model;

        private final Method mutator;

        private ModelPropertyChangeListener(AbsoluteDmdCoordinateEditorModel model, Method mutator) {
            this.model = model;
            this.mutator = mutator;
        }

        @Override
        public void propertyChange(PropertyChangeEvent evt) {
            if (!valueIsAdjusting) {
                Object newValue = evt.getNewValue();

                try {

                    valueModelIsAdjusting = true;
                    try {
                        mutator.invoke(model.getBean(), newValue);
                    } finally {
                        valueModelIsAdjusting = false;
                    }

                } catch (Exception e) {
                    throw new RuntimeException(e);
                }
            }
        }
    }
}
